@@ -2,7 +2,7 @@
                           diff.cpp  -  description
                              -------------------
     begin                : Mon Mar 18 2002
-    copyright            : (C) 2002-2004 by Joachim Eibl
+    copyright            : (C) 2002-2007 by Joachim Eibl
     email                : joachim.eibl at gmx.de
  ***************************************************************************/
 
@@ -15,6 +15,7 @@
  *                                                                         *
  ***************************************************************************/
 
+#include <cstdlib>
 #include <stdio.h>
 #include <iostream>
 
@@ -161,6 +162,7 @@ SourceData::~SourceData()
 
 void SourceData::reset()
 {
+   m_pEncoding = 0;
    m_fileAccess = FileAccess();
    m_normalData.reset();
    m_lmppData.reset();
@@ -291,7 +293,8 @@ bool SourceData::isFromBuffer()
 
 bool SourceData::isBinaryEqualWith( const SourceData& other ) const
 {
-   return getSizeBytes() == other.getSizeBytes() && 
+   return m_fileAccess.exists() && other.m_fileAccess.exists() &&
+          getSizeBytes() == other.getSizeBytes() && 
           ( getSizeBytes()==0 || memcmp( getBuf(), other.getBuf(), getSizeBytes() )==0 );
 }
 
@@ -358,6 +361,7 @@ static bool convertFileEncoding( const QString& fileNameIn, QTextCodec* pCodecIn
       return false;
    QTextStream inStream( &in );
    inStream.setCodec( pCodecIn );
+   //inStream.setAutoDetectUnicode( false ); //not available in Qt3, will always detect UCS2
 
    QFile out( fileNameOut );
    if ( ! out.open( IO_WriteOnly ) )
@@ -371,15 +375,49 @@ static bool convertFileEncoding( const QString& fileNameIn, QTextCodec* pCodecIn
    return true;
 }
 
-void SourceData::readAndPreprocess(QTextCodec* pEncoding)
+static QTextCodec* detectEncoding( const char* buf, long size, long& skipBytes )
+{
+   if (size>=2)
+   {
+      skipBytes = 0; // In Qt3 UTF-16LE can only be used if autodetected.
+      if (buf[0]=='\xFF' && buf[1]=='\xFE' )
+         return QTextCodec::codecForName( "ISO-10646-UCS2" );// "UTF-16LE"
+
+      if (buf[0]=='\xFE' && buf[1]=='\xFF' )
+         return QTextCodec::codecForName( "ISO-10646-UCS2" );// "UTF-16BE". Qt3 autodetects the difference but has no name for it.
+   }
+   if (size>=3)
+   {
+      skipBytes = 3;
+      if (buf[0]=='\xEF' && buf[1]=='\xBB' && buf[2]=='\xBF' )
+         return QTextCodec::codecForName( "UTF-8-BOM" );
+   }
+   skipBytes = 0;
+   return 0;
+}
+
+QTextCodec* SourceData::detectEncoding( const QString& fileName, QTextCodec* pFallbackCodec )
+{
+   QFile f(fileName);
+   if ( f.open(IO_ReadOnly) )
+   {
+      char buf[4];
+      long size = f.readBlock( buf, sizeof(buf) );
+      long skipBytes = 0;
+      QTextCodec* pCodec = ::detectEncoding( buf, size, skipBytes );
+      if (pCodec)
+         return pCodec;
+   }
+   return pFallbackCodec;
+}
+
+void SourceData::readAndPreprocess( QTextCodec* pEncoding, bool bAutoDetectUnicode )
 {
    m_pEncoding = pEncoding;
    QString fileNameIn1;
    QString fileNameOut1;
    QString fileNameIn2;
    QString fileNameOut2;
-   QTextCodec* pEncoding1 = pEncoding;
-   QTextCodec* pEncoding2 = pEncoding;
 
    bool bTempFileFromClipboard = !m_fileAccess.isValid();
 
@@ -397,14 +435,18 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding)
          m_fileAccess.copyFile(m_tempInputFileName);
          fileNameIn1 = m_tempInputFileName;
       }
+      if ( bAutoDetectUnicode )
+      {
+         m_pEncoding = detectEncoding( fileNameIn1, pEncoding );
+      }
    }
    else // The input was set via setData(), probably from clipboard.
    {
       fileNameIn1 = m_tempInputFileName;
       m_pEncoding = QTextCodec::codecForName("UTF-8");
-      pEncoding1 = m_pEncoding;
-      pEncoding2 = m_pEncoding;
    }
+   QTextCodec* pEncoding1 = m_pEncoding;
+   QTextCodec* pEncoding2 = m_pEncoding;
 
    m_normalData.reset();
    m_lmppData.reset();
@@ -469,7 +511,7 @@ void SourceData::readAndPreprocess(QTextCodec* pEncoding)
          pEncoding2 = pEncoding1;
          if ( pEncoding2 != m_pOptionDialog->m_pEncodingPP )
          {
-            // Before running the preprocessor convert to the format that the preprocesor expects.
+            // Before running the preprocessor convert to the format that the preprocessor expects.
             fileNameInPP = FileAccess::tempFileName();
             pEncoding2 = m_pOptionDialog->m_pEncodingPP;
             convertFileEncoding( fileNameIn2, pEncoding1, fileNameInPP, pEncoding2 );
@@ -564,16 +606,20 @@ void SourceData::FileData::preprocess( bool bPreserveCR, QTextCodec* pEncoding )
 {
    //m_unicodeBuf = decodeString( m_pBuf, m_size, eEncoding );
 
+   long skipBytes = 0;
+   QTextCodec* pCodec = ::detectEncoding( m_pBuf, m_size, skipBytes );
+   if ( pCodec != pEncoding )
+      skipBytes=0;
+
    QByteArray ba;
-   ba.setRawData( m_pBuf, m_size );
+   ba.setRawData( m_pBuf+skipBytes, m_size-skipBytes );
    QTextStream ts( ba, IO_ReadOnly );
    ts.setCodec( pEncoding);
+   //ts.setAutoDetectUnicode( false );
    m_unicodeBuf = ts.read();
-   ba.resetRawData( m_pBuf, m_size );
+   ba.resetRawData( m_pBuf+skipBytes, m_size-skipBytes );
 
    int ucSize = m_unicodeBuf.length();
-   m_unicodeBuf += "        ";  // Some extra bytes at the end of the buffer are needed by
-                                // the diff algorithm. See also GnuDiff::diff_2_files().
    const QChar* p = m_unicodeBuf.unicode();
 
    m_bIsText = true;
