@@ -2,7 +2,7 @@
                           diff.h  -  description
                              -------------------
     begin                : Mon Mar 18 2002
-    copyright            : (C) 2002-2004 by Joachim Eibl
+    copyright            : (C) 2002-2007 by Joachim Eibl
     email                : joachim.eibl at gmx.de
  ***************************************************************************/
 
@@ -29,8 +29,8 @@
 #include <assert.h>
 #include "common.h"
 #include "fileaccess.h"
-#include "optiondialog.h"
 
+class OptionDialog;
 
 // Each range with matching elements is followed by a range with differences on either side.
 // Then again range of matching elements should follow.
@@ -259,7 +259,7 @@ public:
 
    void setFilename(const QString& filename);
    void setFileAccess( const FileAccess& fa );
-   FileAccess& getFileAccess();
+   //FileAccess& getFileAccess();
    QString getFilename();
    void setAliasName(const QString& a);
    QString getAliasName();
@@ -270,14 +270,17 @@ public:
    void setData( const QString& data );
    bool isValid(); // Either no file is specified or reading was successful
 
-   void readAndPreprocess(QTextCodec* pEncoding);
+   void readAndPreprocess(QTextCodec* pEncoding, bool bAutoDetectUnicode );
    bool saveNormalDataAs( const QString& fileName );
 
    bool isBinaryEqualWith( const SourceData& other ) const;
 
    void reset();
 
+   QTextCodec* getEncoding() const { return m_pEncoding; }
+
 private:
+   QTextCodec* detectEncoding( const QString& fileName, QTextCodec* pFallbackCodec );
    QString m_aliasName;
    FileAccess m_fileAccess;
    OptionDialog* m_pOptionDialog;
