@@ -2,7 +2,7 @@
                           difftextwindow.cpp  -  description
                              -------------------
     begin                : Mon Apr 8 2002
-    copyright            : (C) 2002-2006 by Joachim Eibl
+    copyright            : (C) 2002-2007 by Joachim Eibl
     email                : joachim.eibl at gmx.de
  ***************************************************************************/
 
@@ -32,6 +32,7 @@
 #include <qtextcodec.h>
 #include <optiondialog.h>
 #include <math.h>
+#include <cstdlib>
 #include <qdragobject.h>
 #include <klocale.h>
 #include <kfiledialog.h>
@@ -467,11 +468,15 @@ void DiffTextWindow::mouseDoubleClickEvent( QMouseEvent* e )
       QString s;
       if ( d->m_bWordWrap )
       {
+         if ( line<0 || line >= (int)d->m_diff3WrapLineVector.size() )
+            return;
          const Diff3WrapLine& d3wl = d->m_diff3WrapLineVector[line];
          s = d->getString( d3wl.diff3LineIndex ).mid( d3wl.wrapLineOffset, d3wl.wrapLineLength );
       }
       else
       {
+         if ( line<0 || line >= (int)d->m_pDiff3LineVector->size() )
+            return;
          s = d->getString( line );
       }
 
@@ -1072,6 +1077,8 @@ void DiffTextWindowData::draw( MyPainter& p, const QRect& invalidRect, int devic
 
 QString DiffTextWindowData::getString( int d3lIdx )
 {
+   if ( d3lIdx<0 || d3lIdx>=(int)m_pDiff3LineVector->size() )
+      return QString();
    const Diff3Line* d3l = (*m_pDiff3LineVector)[d3lIdx];
    DiffList* pFineDiff1;
    DiffList* pFineDiff2;
@@ -1608,7 +1615,7 @@ void DiffTextWindowFrame::init()
    if ( pDTW )
    {
       QString s = pDTW->d->m_filename ;
-      d->m_pFileSelection->setText( s );
+      d->m_pFileSelection->setText( QDir::convertSeparators(s) );
       QString winId = pDTW->d->m_winIdx==1 ? 
                              ( pDTW->d->m_bTriple?"A (Base)":"A") :
                              ( pDTW->d->m_winIdx==2 ? "B" : "C" );
