@@ -2,7 +2,7 @@
                           kreplacements.cpp  -  description
                              -------------------
     begin                : Sat Aug 3 2002
-    copyright            : (C) 2002-2006 by Joachim Eibl
+    copyright            : (C) 2002-2007 by Joachim Eibl
     email                : joachim.eibl at gmx.de
  ***************************************************************************/
 
@@ -91,7 +91,7 @@ static void showHelp()
       else
       {
          QFileInfo prog( buf );
-         _spawnlp( _P_NOWAIT , prog.filePath().ascii(), prog.fileName().ascii(), ("file:///"+helpFile.absFilePath()).ascii(), NULL );
+         _spawnlp( _P_NOWAIT , prog.filePath().ascii(), prog.fileName().ascii(), ("\"file:///"+helpFile.absFilePath()+"\"").ascii(), NULL );
       }
 
    #else
