@@ -2,7 +2,7 @@
                           mergeresultwindow.cpp  -  description
                              -------------------
     begin                : Sun Apr 14 2002
-    copyright            : (C) 2002-2006 by Joachim Eibl
+    copyright            : (C) 2002-2007 by Joachim Eibl
     email                : joachim.eibl at gmx.de
  ***************************************************************************/
 
@@ -27,6 +27,12 @@
 #include <qpopupmenu.h>
 #include <qstatusbar.h>
 #include <qregexp.h>
+#include <qlabel.h>
+#include <qlineedit.h>
+#include <qcombobox.h>
+#include <qlayout.h>
+#include <qtextcodec.h>
+#include <qdragobject.h>
 
 #include <klocale.h>
 #include <kmessagebox.h>
@@ -58,7 +64,6 @@ MergeResultWindow::MergeResultWindow(
    m_bModified = false;
    m_eOverviewMode=Overview::eOMNormal;
 
-   m_fileName = "";
    m_pldA = 0;
    m_pldB = 0;
    m_pldC = 0;
@@ -92,8 +97,7 @@ void MergeResultWindow::init(
    const LineData* pLineDataB, int sizeB,
    const LineData* pLineDataC, int sizeC,
    const Diff3LineList* pDiff3LineList,
-   TotalDiffStatus* pTotalDiffStatus,
-   QString fileName
+   TotalDiffStatus* pTotalDiffStatus
    )
 {
    m_firstLine = 0;
@@ -104,9 +108,8 @@ void MergeResultWindow::init(
    m_bInsertMode = true;
    m_scrollDeltaX = 0;
    m_scrollDeltaY = 0;
-   m_bModified = false;
+   setModified( false );
    
-   m_fileName = fileName;
    m_pldA = pLineDataA;
    m_pldB = pLineDataB;
    m_pldC = pLineDataC;
@@ -141,7 +144,6 @@ void MergeResultWindow::reset()
    m_pldA = 0;
    m_pldB = 0;
    m_pldC = 0;
-   m_fileName = "";
 }
 
 // Calculate the merge information for the given Diff3Line.
@@ -476,7 +478,7 @@ void MergeResultWindow::merge(bool bAutoSolve, int defaultSelector, bool bConfli
    //m_firstLine = 0; // Must not set line/column without scrolling there
    //m_firstColumn = 0;
 
-   m_bModified = false;
+   setModified(false);
 
    m_currentMergeLineIt = m_mergeLineList.begin();
    slotGoTop();
@@ -559,38 +561,38 @@ void MergeResultWindow::go( e_Direction eDir, e_EndPoint eEndPoint )
       if (eDir==eUp) i = m_mergeLineList.begin();     // first mergeline
       else           i = --m_mergeLineList.end();     // last mergeline
 
-      while ( i!=m_mergeLineList.end() && ! i->bDelta )
+      while ( isItAtEnd(eDir==eUp, i) && ! i->bDelta )
       {
          if ( eDir==eUp )  ++i;                       // search downwards
          else              --i;                       // search upwards
       }
    }
-   else if ( eEndPoint == eDelta  &&  i!=m_mergeLineList.end())
+   else if ( eEndPoint == eDelta  &&  isItAtEnd(eDir!=eUp, i) )
    {
       do
       {
          if ( eDir==eUp )  --i;
          else              ++i;
       }
-      while ( i!=m_mergeLineList.end() && ( i->bDelta == false || checkOverviewIgnore(i) || bSkipWhiteConflicts && i->bWhiteSpaceConflict ) );
+      while ( isItAtEnd(eDir!=eUp, i) && ( i->bDelta == false || checkOverviewIgnore(i) || bSkipWhiteConflicts && i->bWhiteSpaceConflict ) );
    }
-   else if ( eEndPoint == eConflict  &&  i!=m_mergeLineList.end() )
+   else if ( eEndPoint == eConflict  &&  isItAtEnd(eDir!=eUp, i) )
    {
       do
       {
          if ( eDir==eUp )  --i;
          else              ++i;
       }
-      while ( i!=m_mergeLineList.end() && (i->bConflict == false || bSkipWhiteConflicts && i->bWhiteSpaceConflict ) );
+      while ( isItAtEnd(eDir!=eUp, i) && (i->bConflict == false || bSkipWhiteConflicts && i->bWhiteSpaceConflict ) );
    }
-   else if ( i!=m_mergeLineList.end()  &&  eEndPoint == eUnsolvedConflict )
+   else if ( isItAtEnd(eDir!=eUp, i)  &&  eEndPoint == eUnsolvedConflict )
    {
       do
       {
          if ( eDir==eUp )  --i;
          else              ++i;
       }
-      while ( i!=m_mergeLineList.end() && ! i->mergeEditLineList.begin()->isConflict() );
+      while ( isItAtEnd(eDir!=eUp, i) && ! i->mergeEditLineList.begin()->isConflict() );
    }
 
    if ( isVisible() )
@@ -608,7 +610,7 @@ bool MergeResultWindow::isDeltaAboveCurrent()
    do
    {
       --i;
-      if ( i->bDelta && !( bSkipWhiteConflicts && i->bWhiteSpaceConflict ) ) return true;
+      if ( i->bDelta && !checkOverviewIgnore(i) && !( bSkipWhiteConflicts && i->bWhiteSpaceConflict ) ) return true;
    }
    while (i!=m_mergeLineList.begin());
 
@@ -626,7 +628,7 @@ bool MergeResultWindow::isDeltaBelowCurrent()
       ++i;
       for( ; i!=m_mergeLineList.end(); ++i )
       {
-         if ( i->bDelta && !( bSkipWhiteConflicts && i->bWhiteSpaceConflict ) ) return true;
+         if ( i->bDelta && !checkOverviewIgnore(i) && !( bSkipWhiteConflicts && i->bWhiteSpaceConflict ) ) return true;
       }
    }
    return false;
@@ -638,10 +640,12 @@ bool MergeResultWindow::isConflictAboveCurrent()
    MergeLineList::iterator i = m_currentMergeLineIt;
    if (i == m_mergeLineList.begin()) return false;
 
+   bool bSkipWhiteConflicts = ! m_pOptionDialog->m_bShowWhiteSpace;
+
    do
    {
       --i;
-      if ( i->bConflict ) return true;
+      if ( i->bConflict && !(bSkipWhiteConflicts && i->bWhiteSpaceConflict) ) return true;
    } 
    while (i!=m_mergeLineList.begin());
    
@@ -652,18 +656,27 @@ bool MergeResultWindow::isConflictBelowCurrent()
 {
    MergeLineList::iterator i = m_currentMergeLineIt;
    if (m_mergeLineList.empty()) return false;
-   
+
+   bool bSkipWhiteConflicts = ! m_pOptionDialog->m_bShowWhiteSpace;
+
    if (i!=m_mergeLineList.end())
    {
       ++i;
       for( ; i!=m_mergeLineList.end(); ++i )
       {
-         if ( i->bConflict ) return true;
+         if ( i->bConflict && !(bSkipWhiteConflicts && i->bWhiteSpaceConflict) ) return true;
       }
    }
    return false;
 }
 
+bool MergeResultWindow::isUnsolvedConflictAtCurrent()
+{
+   if (m_mergeLineList.empty()) return false;
+   MergeLineList::iterator i = m_currentMergeLineIt;
+   return i->mergeEditLineList.begin()->isConflict();
+}
+
 bool MergeResultWindow::isUnsolvedConflictAboveCurrent()
 {
    if (m_mergeLineList.empty()) return false;
@@ -796,12 +809,12 @@ void MergeResultWindow::showNrOfConflicts()
    else  if ( m_pTotalDiffStatus->bTextAEqB && m_pTotalDiffStatus->bTextAEqC )
       totalInfo += i18n("All input files contain the same text.");
    else {
-      if    ( m_pTotalDiffStatus->bBinaryAEqB ) totalInfo += i18n("Files A and B are binary equal.\n");
-      else if ( m_pTotalDiffStatus->bTextAEqB ) totalInfo += i18n("Files A and B have equal text. \n");
-      if    ( m_pTotalDiffStatus->bBinaryAEqC ) totalInfo += i18n("Files A and C are binary equal.\n");
-      else if ( m_pTotalDiffStatus->bTextAEqC ) totalInfo += i18n("Files A and C have equal text. \n");
-      if    ( m_pTotalDiffStatus->bBinaryBEqC ) totalInfo += i18n("Files B and C are binary equal.\n");
-      else if ( m_pTotalDiffStatus->bTextBEqC ) totalInfo += i18n("Files B and C have equal text. \n");
+      if    ( m_pTotalDiffStatus->bBinaryAEqB ) totalInfo += i18n("Files %1 and %2 are binary equal.\n").arg("A").arg("B");
+      else if ( m_pTotalDiffStatus->bTextAEqB ) totalInfo += i18n("Files %1 and %2 have equal text.\n").arg("A").arg("B");
+      if    ( m_pTotalDiffStatus->bBinaryAEqC ) totalInfo += i18n("Files %1 and %2 are binary equal.\n").arg("A").arg("C");
+      else if ( m_pTotalDiffStatus->bTextAEqC ) totalInfo += i18n("Files %1 and %2 have equal text.\n").arg("A").arg("C");
+      if    ( m_pTotalDiffStatus->bBinaryBEqC ) totalInfo += i18n("Files %1 and %2 are binary equal.\n").arg("B").arg("C");
+      else if ( m_pTotalDiffStatus->bTextBEqC ) totalInfo += i18n("Files %1 and %2 have equal text.\n").arg("B").arg("C");
    }
 
    int nrOfUnsolvedConflicts = getNrOfUnsolvedConflicts();
@@ -944,7 +957,7 @@ void MergeResultWindow::chooseGlobal(int selector, bool bConflictsOnly, bool bWh
    resetSelection();
 
    merge( false, selector, bConflictsOnly, bWhiteSpaceOnly );
-   emit modified();
+   setModified( true );
    update();
    int wsc;
    int nofUnsolved = getNrOfUnsolvedConflicts(&wsc);
@@ -956,7 +969,7 @@ void MergeResultWindow::slotAutoSolve()
 {
    resetSelection();
    merge( true, -1 );
-   emit modified();
+   setModified( true );
    update();
    int wsc;
    int nofUnsolved = getNrOfUnsolvedConflicts(&wsc);
@@ -968,7 +981,7 @@ void MergeResultWindow::slotUnsolve()
 {
    resetSelection();
    merge( false, -1 );
-   emit modified();
+   setModified( true );
    update();
    int wsc;
    int nofUnsolved = getNrOfUnsolvedConflicts(&wsc);
@@ -976,11 +989,30 @@ void MergeResultWindow::slotUnsolve()
          .arg(nofUnsolved).arg(wsc) );
 }
 
+static QString calcHistoryLead(const QString& s )
+{
+   // Return the start of the line until the first white char after the first non white char.
+   unsigned int i;
+   for( i=0; i<s.length(); ++i )
+   {
+      if (s[i]!=' ' && s[i]!='\t')
+      {
+         for( ; i<s.length(); ++i )
+         {
+            if (s[i]==' ' || s[i]=='\t')
+            {
+               return s.left(i);
+            }
+         }
+         return s;  // Very unlikely
+      }
+   }
+   return "";  // Must be an empty string, not a null string.
+}
 
-static void findHistoryRange( bool bThreeFiles, const Diff3LineList* pD3LList, 
+static void findHistoryRange( const QRegExp& historyStart, bool bThreeFiles, const Diff3LineList* pD3LList, 
                              Diff3LineList::const_iterator& iBegin, Diff3LineList::const_iterator& iEnd, int& idxBegin, int& idxEnd )
 {
-   QRegExp historyStart(".*\\$Log.*\\$.*");
    QString historyLead;
    // Search for start of history
    for( iBegin = pD3LList->begin(), idxBegin=0; iBegin!=pD3LList->end(); ++iBegin, ++idxBegin )
@@ -989,7 +1021,7 @@ static void findHistoryRange( bool bThreeFiles, const Diff3LineList* pD3LList,
            historyStart.exactMatch( iBegin->getString(B) ) && 
            ( !bThreeFiles || historyStart.exactMatch( iBegin->getString(C) ) ) )
       {
-         historyLead = iBegin->getString(A).section(' ', 0, 0, QString::SectionIncludeLeadingSep);
+         historyLead = calcHistoryLead( iBegin->getString(A) );
          break;
       }
    }
@@ -999,9 +1031,9 @@ static void findHistoryRange( bool bThreeFiles, const Diff3LineList* pD3LList,
       QString sA = iEnd->getString(A);
       QString sB = iEnd->getString(B);
       QString sC = iEnd->getString(C);
-      if ( ! ((sA.isNull() || historyLead == sA.section(' ',0,0,QString::SectionIncludeLeadingSep)) &&
-           (sB.isNull() || historyLead == sB.section(' ',0,0,QString::SectionIncludeLeadingSep)) &&
-           (!bThreeFiles || sC.isNull() || historyLead == sC.section(' ',0,0,QString::SectionIncludeLeadingSep))
+      if ( ! ((sA.isNull() || historyLead == calcHistoryLead(sA) ) &&
+              (sB.isNull() || historyLead == calcHistoryLead(sB) ) &&
+           (!bThreeFiles || sC.isNull() || historyLead == calcHistoryLead(sC) )
          ))
       {
          break; // End of the history
@@ -1100,7 +1132,7 @@ void MergeResultWindow::collectHistoryInformation(
    {
       const LineData* pld = id3l->getLineData(src);
       QString s( pld->pLine, pld->size );
-      historyLead = s.section(' ',0,0,QString::SectionIncludeLeadingSep);
+      historyLead = calcHistoryLead(s);
    }
    QRegExp historyStart = m_pOptionDialog->m_historyStartRegExp;
    ++id3l; // Skip line with "$Log ... $"
@@ -1116,7 +1148,7 @@ void MergeResultWindow::collectHistoryInformation(
       const LineData* pld = id3l->getLineData(src);
       if ( !pld ) continue;
       QString s( pld->pLine, pld->size );
-      if (historyLead.isNull()) historyLead = s.section(' ',0,0,QString::SectionIncludeLeadingSep);
+      if (historyLead.isNull()) historyLead = calcHistoryLead(s);
       QString sLine = s.mid(historyLead.length());
       if ( ( !bUseRegExp && !sLine.stripWhiteSpace().isEmpty() && bPrevLineIsEmpty )
            || bUseRegExp && newHistoryEntry.exactMatch( sLine ) 
@@ -1226,7 +1258,7 @@ void MergeResultWindow::slotMergeHistory()
    int d3lHistoryEndLineIdx = -1;
 
    // Search for history start, history end in the diff3LineList
-   findHistoryRange( m_pldC!=0, m_pDiff3LineList, iD3LHistoryBegin, iD3LHistoryEnd, d3lHistoryBeginLineIdx, d3lHistoryEndLineIdx );
+   findHistoryRange( m_pOptionDialog->m_historyStartRegExp, m_pldC!=0, m_pDiff3LineList, iD3LHistoryBegin, iD3LHistoryEnd, d3lHistoryBeginLineIdx, d3lHistoryEndLineIdx );
 
    if (  iD3LHistoryBegin != m_pDiff3LineList->end() )
    {
@@ -1250,33 +1282,36 @@ void MergeResultWindow::slotMergeHistory()
       bool bHistoryMergeSorting = m_pOptionDialog->m_bHistoryMergeSorting  && ! m_pOptionDialog->m_historyEntryStartSortKeyOrder.isEmpty() && 
                                   ! m_pOptionDialog->m_historyEntryStartRegExp.isEmpty();
 
-      // Remove parts from the historyMap and hitList that stay in place
-      if ( bHistoryMergeSorting )
+      if ( m_pOptionDialog->m_maxNofHistoryEntries==-1 )
       {
-         while ( ! historyMap.empty() )
+         // Remove parts from the historyMap and hitList that stay in place
+         if ( bHistoryMergeSorting )
          {
-            HistoryMap::iterator hMapIt = historyMap.begin();
-            if( hMapIt->second.staysInPlace( m_pldC!=0, iD3LHistoryEnd ) )
-               historyMap.erase(hMapIt);
-            else
-               break;
+            while ( ! historyMap.empty() )
+            {
+               HistoryMap::iterator hMapIt = historyMap.begin();
+               if( hMapIt->second.staysInPlace( m_pldC!=0, iD3LHistoryEnd ) )
+                  historyMap.erase(hMapIt);
+               else
+                  break;
+            }
          }
-      }
-      else
-      {
-         while ( ! hitList.empty() )
+         else
          {
-            HistoryMap::iterator hMapIt = hitList.back();
-            if( hMapIt->second.staysInPlace( m_pldC!=0, iD3LHistoryEnd ) )
-               hitList.pop_back();
-            else
-               break;
+            while ( ! hitList.empty() )
+            {
+               HistoryMap::iterator hMapIt = hitList.back();
+               if( hMapIt->second.staysInPlace( m_pldC!=0, iD3LHistoryEnd ) )
+                  hitList.pop_back();
+               else
+                  break;
+            }
+         }
+         while (iD3LHistoryOrigEnd != iD3LHistoryEnd)
+         {
+            --iD3LHistoryOrigEnd;
+            --d3lHistoryEndLineIdx;
          }
-      }
-      while (iD3LHistoryOrigEnd != iD3LHistoryEnd)
-      {
-         --iD3LHistoryOrigEnd;
-         --d3lHistoryEndLineIdx;
       }
 
       MergeLineList::iterator iMLLStart = splitAtDiff3LineIdx(d3lHistoryBeginLineIdx);
@@ -1295,17 +1330,21 @@ void MergeResultWindow::slotMergeHistory()
       iMLLStart->mergeEditLineList.clear();
       // Now insert the complete history into the first MergeLine of the history
       iMLLStart->mergeEditLineList.push_back( MergeEditLine( iD3LHistoryBegin, m_pldC == 0 ? B : C ) );
-      QString lead = iD3LHistoryBegin->getString(A).section(' ',0,0,QString::SectionIncludeLeadingSep);
+      QString lead = calcHistoryLead( iD3LHistoryBegin->getString(A) );
       MergeEditLine mel( m_pDiff3LineList->end() );
       mel.setString( lead );
       iMLLStart->mergeEditLineList.push_back(mel);
 
+      int historyCount = 0;
       if ( bHistoryMergeSorting )
       {
          // Create a sorted history
          HistoryMap::reverse_iterator hmit;
          for ( hmit = historyMap.rbegin(); hmit != historyMap.rend(); ++hmit )
          {
+            if ( historyCount==m_pOptionDialog->m_maxNofHistoryEntries )
+               break;
+            ++historyCount;
             HistoryMapEntry& hme = hmit->second;
             MergeEditLineList& mell = hme.choice(m_pldC!=0);
             if (!mell.empty())
@@ -1318,6 +1357,9 @@ void MergeResultWindow::slotMergeHistory()
          std::list< HistoryMap::iterator >::iterator hlit;
          for ( hlit = hitList.begin(); hlit != hitList.end(); ++hlit )
          {
+            if ( historyCount==m_pOptionDialog->m_maxNofHistoryEntries )
+               break;
+            ++historyCount;
             HistoryMapEntry& hme = (*hlit)->second;
             MergeEditLineList& mell = hme.choice(m_pldC!=0);
             if (!mell.empty())
@@ -1492,9 +1534,8 @@ QString MergeResultWindow::MergeEditLine::getString( const MergeResultWindow* mr
    if ( ! isModified() )
    {
       int src = m_src;
-      const Diff3Line& d3l = *m_id3l;
       if ( src == 0 )   { return QString(); }
-
+      const Diff3Line& d3l = *m_id3l;
       const LineData* pld = 0;
       assert( src == A || src == B || src == C );
       if      ( src == A && d3l.lineA!=-1 ) pld = &mrw->m_pldA[ d3l.lineA ];
@@ -1562,7 +1603,7 @@ void MergeResultWindow::writeLine(
    int fontWidth = fm.width("W");
    int fontAscent = fm.ascent();
 
-   int topLineYOffset = fontHeight + 3;
+   int topLineYOffset = 0;
    int xOffset = fontWidth * leftInfoWidth;
 
    int yOffset = ( line-m_firstLine ) * fontHeight;
@@ -1726,26 +1767,6 @@ void MergeResultWindow::paintEvent( QPaintEvent*  )
 
       //int visibleLines = height() / fontHeight;
 
-      {  // Draw the topline
-         QString s = " " +i18n("Output") + " : " + m_fileName + " ";
-         if (m_bModified)
-            s += i18n("[Modified]");
-
-         int topLineYOffset = fontHeight + 3;
-
-         if (hasFocus())
-         {
-            p.fillRect( 0, 0, width(), topLineYOffset, Qt::lightGray /*m_pOptionDialog->m_diffBgColor*/ );
-         }
-         else
-         {
-            p.fillRect( 0, 0, width(), topLineYOffset, m_pOptionDialog->m_bgColor );
-         }
-         p.setPen( m_pOptionDialog->m_fgColor );
-         p.QPainter::drawText( 0, fontAscent+1, s );
-         p.drawLine( 0, fontHeight + 2, width(), fontHeight + 2 );
-      }
-
       int lastVisibleLine = m_firstLine + getNofVisibleLines() + 5;
       int nofColumns = 0;
       int line = 0;
@@ -1801,7 +1822,7 @@ void MergeResultWindow::paintEvent( QPaintEvent*  )
 
    QPainter painter(this);
 
-   int topLineYOffset = fontHeight + 3;
+   int topLineYOffset = 0;
    int xOffset = fontWidth * leftInfoWidth;
    int yOffset = ( m_cursorYPos - m_firstLine ) * fontHeight + topLineYOffset;
    int xCursor = ( m_cursorXPos - m_firstColumn ) * fontWidth + xOffset;
@@ -1823,7 +1844,7 @@ void MergeResultWindow::paintEvent( QPaintEvent*  )
    if ( m_bCursorOn && hasFocus() && m_cursorYPos>=m_firstLine )
    {
       MyPainter painter(this, m_pOptionDialog->m_bRightToLeftLanguage, width(), fontWidth);
-      int topLineYOffset = fontHeight + 3;
+      int topLineYOffset = 0;
       int xOffset = fontWidth * leftInfoWidth;
 
       int yOffset = ( m_cursorYPos-m_firstLine ) * fontHeight + topLineYOffset;
@@ -1889,7 +1910,7 @@ void MergeResultWindow::convertToLinePos( int x, int y, int& line, int& pos )
    int fontHeight = fm.height();
    int fontWidth = fm.width('W');
    int xOffset = (leftInfoWidth-m_firstColumn)*fontWidth;
-   int topLineYOffset = fontHeight + 3;
+   int topLineYOffset = 0;
 
    int yOffset = topLineYOffset - m_firstLine * fontHeight;
 
@@ -2041,9 +2062,8 @@ void MergeResultWindow::mouseMoveEvent ( QMouseEvent * e )
 
       // Scroll because mouse moved out of the window
       const QFontMetrics& fm = fontMetrics();
-      int fontHeight = fm.height();
       int fontWidth = fm.width('W');
-      int topLineYOffset = fontHeight + 3;
+      int topLineYOffset = 0;
       int deltaX=0;
       int deltaY=0;
       if ( ! m_pOptionDialog->m_bRightToLeftLanguage )
@@ -2079,7 +2099,7 @@ void MergeResultWindow::slotCursorUpdate()
 
       const QFontMetrics& fm = fontMetrics();
       int fontWidth = fm.width("W");
-      int topLineYOffset = fm.height() + 3;
+      int topLineYOffset = 0;
       int xOffset = fontWidth * leftInfoWidth;
       int yOffset = ( m_cursorYPos - m_firstLine ) * fm.height() + topLineYOffset;
       int xCursor = ( m_cursorXPos - m_firstColumn ) * fontWidth + xOffset;
@@ -2221,10 +2241,13 @@ void MergeResultWindow::keyPressEvent( QKeyEvent* e )
                      break;
                   }
                }
-               --melIt1;
-               if ( melIt1 == mlIt1->mergeEditLineList.end() ) {
+               // Go back one line
+               if ( melIt1 != mlIt1->mergeEditLineList.begin() )
+                  --melIt1;
+               else
+               {
+                  if ( mlIt1 == m_mergeLineList.begin() ) break;
                   --mlIt1;
-                  if ( mlIt1 == m_mergeLineList.end() ) break;
                   melIt1 = mlIt1->mergeEditLineList.end();
                   --melIt1;
                }
@@ -2283,10 +2306,10 @@ void MergeResultWindow::keyPressEvent( QKeyEvent* e )
          }
          break;
 
-      case  Qt::Key_Up:       --y;                     bYMoveKey=true;   break;
-      case  Qt::Key_Down:     ++y;                     bYMoveKey=true;    break;
-      case  Qt::Key_PageUp:   y-=getNofVisibleLines(); bYMoveKey=true;    break;
-      case  Qt::Key_PageDown: y+=getNofVisibleLines(); bYMoveKey=true;    break;
+      case  Qt::Key_Up:       if (!bCtrl){ --y;                     bYMoveKey=true; }  break;
+      case  Qt::Key_Down:     if (!bCtrl){ ++y;                     bYMoveKey=true; }  break;
+      case  Qt::Key_PageUp:   if (!bCtrl){ y-=getNofVisibleLines(); bYMoveKey=true; }  break;
+      case  Qt::Key_PageDown: if (!bCtrl){ y+=getNofVisibleLines(); bYMoveKey=true; }  break;
       default:
       {
          QString t = e->text();
@@ -2635,20 +2658,18 @@ void MergeResultWindow::resetSelection()
    update();
 }
 
-void MergeResultWindow::setModified()
+void MergeResultWindow::setModified(bool bModified)
 {
-   if (!m_bModified)
+   if (bModified != m_bModified)
    {
-      m_bModified = true;
-      emit modified();
+      m_bModified = bModified;
+      emit modifiedChanged(m_bModified);
    }
 }
 
 /// Saves and returns true when successful.
-bool MergeResultWindow::saveDocument( const QString& fileName )
+bool MergeResultWindow::saveDocument( const QString& fileName, QTextCodec* pEncoding )
 {
-   m_fileName = fileName;
-
    // Are still conflicts somewhere?
    if ( getNrOfUnsolvedConflicts()>0 )
    {
@@ -2674,7 +2695,7 @@ bool MergeResultWindow::saveDocument( const QString& fileName )
 
    QByteArray dataArray;
    QTextStream textOutStream(dataArray, IO_WriteOnly);
-   textOutStream.setCodec( m_pOptionDialog->m_pEncodingOut );
+   textOutStream.setCodec( pEncoding );
 
    int line = 0;
    MergeLineList::iterator mlIt = m_mergeLineList.begin();
@@ -2699,9 +2720,8 @@ bool MergeResultWindow::saveDocument( const QString& fileName )
             }
 
             textOutStream << str;
+            ++line;
          }
-
-         ++line;
       }
    }
    bool bSuccess = file.writeFile( dataArray.data(), dataArray.size() );
@@ -2711,7 +2731,7 @@ bool MergeResultWindow::saveDocument( const QString& fileName )
       return false;
    }
 
-   m_bModified = false;
+   setModified( false );
    update();
 
    return true;
@@ -3028,5 +3048,175 @@ void Overview::paintEvent( QPaintEvent* )
    painter.drawRect( 1, y1, w-1, h1 );
 }
 
+WindowTitleWidget::WindowTitleWidget(OptionDialog* pOptionDialog, QWidget* pParent)
+:QWidget(pParent)
+{
+   m_pOptionDialog = pOptionDialog;
+   //setAutoFillBackground(true);
+
+   QHBoxLayout* pHLayout = new QHBoxLayout(this);
+   pHLayout->setMargin(2);
+   pHLayout->setSpacing(2);
+
+   m_pLabel = new QLabel(i18n("Output")+":", this);
+   pHLayout->addWidget( m_pLabel );
+
+   m_pFileNameLineEdit = new QLineEdit(this);
+   pHLayout->addWidget( m_pFileNameLineEdit, 6 );
+   m_pFileNameLineEdit->installEventFilter( this );
+   m_pFileNameLineEdit->setReadOnly( true );
+
+   //m_pBrowseButton = new QPushButton("...");
+   //pHLayout->addWidget( m_pBrowseButton, 0 );
+   //connect( m_pBrowseButton, SIGNAL(clicked()), this, SLOT(slotBrowseButtonClicked()));
+
+   m_pModifiedLabel = new QLabel(i18n("[Modified]"),this);
+   pHLayout->addWidget( m_pModifiedLabel );
+   m_pModifiedLabel->setMinimumSize( m_pModifiedLabel->sizeHint() );
+   m_pModifiedLabel->setText("");
+
+   pHLayout->addStretch(1);
+
+   m_pEncodingLabel = new QLabel(i18n("Encoding for saving")+":",this);
+   pHLayout->addWidget( m_pEncodingLabel );
+
+   m_pEncodingSelector = new QComboBox(this);
+   pHLayout->addWidget( m_pEncodingSelector, 3 );
+   setEncodings(0,0,0);
+}
+
+void WindowTitleWidget::setFileName( const QString& fileName )
+{
+   m_pFileNameLineEdit->setText( QDir::convertSeparators(fileName) );
+}
+
+QString WindowTitleWidget::getFileName()
+{
+   return m_pFileNameLineEdit->text();
+}
+
+void WindowTitleWidget::setEncodings( QTextCodec* pCodecForA, QTextCodec* pCodecForB, QTextCodec* pCodecForC )
+{
+   m_pEncodingSelector->clear();
+   m_codecMap.clear();
+
+   // First sort codec names:
+   std::map<QString, QTextCodec*> names;
+   int i;
+   for(i=0;;++i)
+   {
+       QTextCodec* c = QTextCodec::codecForIndex(i);
+       if ( c==0 ) break;
+       else  names[QString(c->name())]=c;
+   }
+
+   i=0;
+   if ( pCodecForA )
+   {
+      m_pEncodingSelector->insertItem( i18n("Codec from") + " A: " + pCodecForA->name(), i );
+      m_codecMap[i]=pCodecForA;
+      ++i;
+   }
+   if ( pCodecForB )
+   {
+      m_pEncodingSelector->insertItem( i18n("Codec from") + " B: " + pCodecForB->name(), i );
+      m_codecMap[i]=pCodecForB;
+      ++i;
+   }
+   if ( pCodecForC )
+   {
+      m_pEncodingSelector->insertItem( i18n("Codec from") + " C: " + pCodecForC->name(), i );
+      m_codecMap[i]=pCodecForC;
+      ++i;
+   }
+
+   std::map<QString, QTextCodec*>::iterator it;
+   for(it=names.begin();it!=names.end();++it)
+   {
+      m_pEncodingSelector->insertItem( it->first, i );
+      m_codecMap[i]=it->second;
+      ++i;
+   }
+   m_pEncodingSelector->setMinimumSize( m_pEncodingSelector->sizeHint() );
+
+   if ( pCodecForC && pCodecForB && pCodecForA )
+   {
+      if ( pCodecForA == pCodecForB )
+         m_pEncodingSelector->setCurrentItem( 2 ); // C
+      else if ( pCodecForA == pCodecForC )
+         m_pEncodingSelector->setCurrentItem( 1 ); // B
+      else
+         m_pEncodingSelector->setCurrentItem( 2 ); // C
+   }
+   else if ( pCodecForA && pCodecForB )
+      m_pEncodingSelector->setCurrentItem( 1 ); // B
+   else
+      m_pEncodingSelector->setCurrentItem( 0 );
+}
+
+QTextCodec* WindowTitleWidget::getEncoding()
+{
+   return m_codecMap[ m_pEncodingSelector->currentItem() ];
+}
+
+void WindowTitleWidget::setEncoding(QTextCodec* pEncoding)
+{
+   m_pEncodingSelector->setCurrentText( QString( pEncoding->name() ) );
+}
+
+//void WindowTitleWidget::slotBrowseButtonClicked()
+//{
+//   QString current = m_pFileNameLineEdit->text();
+//
+//   KURL newURL = KFileDialog::getSaveURL( current, 0, this, i18n("Select file (not saving yet)"));
+//   if ( !newURL.isEmpty() )
+//   {
+//      m_pFileNameLineEdit->setText( newURL.url() ); 
+//   }
+//}
+
+void WindowTitleWidget::slotSetModified( bool bModified )
+{
+   m_pModifiedLabel->setText( bModified ? i18n("[Modified]") : "" );
+}
+
+bool WindowTitleWidget::eventFilter( QObject* o, QEvent* e )
+{
+   if ( e->type()==QEvent::FocusIn || e->type()==QEvent::FocusOut )
+   {
+      QPalette p = m_pLabel->palette();
+
+      QColor c1 = m_pOptionDialog->m_fgColor;
+      QColor c2 = Qt::lightGray;
+      if ( e->type()==QEvent::FocusOut )
+         c2 = m_pOptionDialog->m_bgColor;
+
+      p.setColor(QColorGroup::Background, c2);
+      setPalette( p );
+
+      p.setColor(QColorGroup::Foreground, c1);
+      m_pLabel->setPalette( p );
+      m_pEncodingLabel->setPalette( p );
+      m_pEncodingSelector->setPalette( p );
+   }
+   if (o == m_pFileNameLineEdit && e->type()==QEvent::Drop)
+   {
+      QDropEvent* d = static_cast<QDropEvent*>(e);
+
+      if ( QUriDrag::canDecode( d ) ) 
+      {
+         QStringList lst;
+         QUriDrag::decodeLocalFiles( d, lst );
+
+         if ( lst.count() > 0 )
+         {
+            static_cast<QLineEdit*>(o)->setText( lst[0] );
+            static_cast<QLineEdit*>(o)->setFocus();
+            return true;
+         }
+      }
+   }
+   return false;
+}
 
 #include "mergeresultwindow.moc"
