@@ -1,6 +1,6 @@
 /*
  *   kdiff3 - Text Diff And Merge Tool
- *   Copyright (C) 2002-2006  Joachim Eibl, joachim.eibl at gmx.de
+ *   Copyright (C) 2002-2007  Joachim Eibl, joachim.eibl at gmx.de
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@
 #include <qtextcodec.h>
 #include <qradiobutton.h>
 #include <qvbuttongroup.h>
+#include <qsettings.h>
 
 #include <kapplication.h>
 #include <kcolorbtn.h>
@@ -460,6 +461,7 @@ OptionDialog::OptionDialog( bool bShowDirMergeSettings, QWidget *parent, char *n
    setupColorPage();
    setupEditPage();
    setupDiffPage();
+   setupMergePage();
    setupOtherOptions();
    if (bShowDirMergeSettings)
       setupDirectoryMergePage();
@@ -495,6 +497,7 @@ void OptionDialog::setupOtherOptions()
    new OptionNum( (int)KToolBar::Top, "ToolBarPos", &m_toolBarPos, this );
    new OptionSize( QSize(600,400),"Geometry", &m_geometry, this );
    new OptionPoint( QPoint(0,22), "Position", &m_position, this );
+   new OptionToggleAction( false, "WindowStateMaximised", &m_bMaximised, this );
 
    new OptionStringList( "RecentAFiles", &m_recentAFiles, this );
    new OptionStringList( "RecentBFiles", &m_recentBFiles, this );
@@ -726,7 +729,7 @@ void OptionDialog::setupEditPage( void )
 
 void OptionDialog::setupDiffPage( void )
 {
-   QFrame *page = addPage( i18n("Diff and Merge"), i18n("Diff and Merge Settings"),
+   QFrame *page = addPage( i18n("Diff"), i18n("Diff Settings"),
                            BarIcon("misc", KIcon::SizeMedium ) );
    QVBoxLayout *topLayout = new QVBoxLayout( page, 5, spacingHint() );
 
@@ -737,15 +740,16 @@ void OptionDialog::setupDiffPage( void )
 
    QLabel* label=0;
 
-   OptionCheckBox* pPreserveCarriageReturn = new OptionCheckBox( i18n("Preserve carriage return"), false, "PreserveCarriageReturn", &m_bPreserveCarriageReturn, page, this );
-   gbox->addMultiCellWidget( pPreserveCarriageReturn, line, line, 0, 1 );
-   QToolTip::add( pPreserveCarriageReturn, i18n(
-      "Show carriage return characters '\\r' if they exist.\n"
-      "Helps to compare files that were modified under different operating systems.")
-      );
-   ++line;
+//   OptionCheckBox* pPreserveCarriageReturn = new OptionCheckBox( i18n("Preserve carriage return"), false, "PreserveCarriageReturn", &m_bPreserveCarriageReturn, page, this );
+//   gbox->addMultiCellWidget( pPreserveCarriageReturn, line, line, 0, 1 );
+//   QToolTip::add( pPreserveCarriageReturn, i18n(
+//      "Show carriage return characters '\\r' if they exist.\n"
+//      "Helps to compare files that were modified under different operating systems.")
+//      );
+//   ++line;
+   QString treatAsWhiteSpace = " ("+i18n("Treat as white space.")+")";
 
-   OptionCheckBox* pIgnoreNumbers = new OptionCheckBox( i18n("Ignore numbers"), false, "IgnoreNumbers", &m_bIgnoreNumbers, page, this );
+   OptionCheckBox* pIgnoreNumbers = new OptionCheckBox( i18n("Ignore numbers")+treatAsWhiteSpace, false, "IgnoreNumbers", &m_bIgnoreNumbers, page, this );
    gbox->addMultiCellWidget( pIgnoreNumbers, line, line, 0, 1 );
    QToolTip::add( pIgnoreNumbers, i18n(
       "Ignore number characters during line matching phase. (Similar to Ignore white space.)\n"
@@ -753,13 +757,13 @@ void OptionDialog::setupDiffPage( void )
       );
    ++line;
 
-   OptionCheckBox* pIgnoreComments = new OptionCheckBox( i18n("Ignore C/C++ comments"), false, "IgnoreComments", &m_bIgnoreComments, page, this );
+   OptionCheckBox* pIgnoreComments = new OptionCheckBox( i18n("Ignore C/C++ comments")+treatAsWhiteSpace, false, "IgnoreComments", &m_bIgnoreComments, page, this );
    gbox->addMultiCellWidget( pIgnoreComments, line, line, 0, 1 );
    QToolTip::add( pIgnoreComments, i18n( "Treat C/C++ comments like white space.")
       );
    ++line;
 
-   OptionCheckBox* pIgnoreCase = new OptionCheckBox( i18n("Ignore case"), false, "IgnoreCase", &m_bIgnoreCase, page, this );
+   OptionCheckBox* pIgnoreCase = new OptionCheckBox( i18n("Ignore case")+treatAsWhiteSpace, false, "IgnoreCase", &m_bIgnoreCase, page, this );
    gbox->addMultiCellWidget( pIgnoreCase, line, line, 0, 1 );
    QToolTip::add( pIgnoreCase, i18n(
       "Treat case differences like white space changes. ('a'<=>'A')")
@@ -788,6 +792,24 @@ void OptionDialog::setupDiffPage( void )
       );
    ++line;
 
+   topLayout->addStretch(10);
+}
+
+void OptionDialog::setupMergePage( void )
+{
+   QFrame *page = addPage( i18n("Merge"), i18n("Merge Settings"),
+                           BarIcon("misc", KIcon::SizeMedium ) );
+   QVBoxLayout *topLayout = new QVBoxLayout( page );
+   topLayout->setMargin( 5 );
+   topLayout->setSpacing( spacingHint() );
+
+   QGridLayout *gbox = new QGridLayout();
+   gbox->setColStretch(1,5);
+   topLayout->addLayout( gbox );
+   int line=0;
+
+   QLabel* label=0;
+
    label = new QLabel( i18n("Auto advance delay (ms):"), page );
    gbox->addWidget( label, line, 0 );
    OptionIntEdit* pAutoAdvanceDelay = new OptionIntEdit( 500, "AutoAdvanceDelay", &m_autoAdvanceDelay, 0, 2000, page, this );
@@ -921,6 +943,13 @@ void OptionDialog::setupDiffPage( void )
       gbox->addMultiCellWidget( m_pHistoryAutoMerge, line, line, 0, 1 );
       QToolTip::add( m_pHistoryAutoMerge, i18n("Run version control history automerge on merge start.") );
       ++line;
+
+      OptionIntEdit* pMaxNofHistoryEntries = new OptionIntEdit( -1, "MaxNofHistoryEntries", &m_maxNofHistoryEntries, -1, 1000, page, this );
+      label = new QLabel( pMaxNofHistoryEntries, i18n("Max number of history entries:"), page );
+      gbox->addWidget( label, line, 0 );
+      gbox->addWidget( pMaxNofHistoryEntries, line, 1 );
+      QToolTip::add( pMaxNofHistoryEntries, i18n("Cut off after specified number. Use -1 for infinite number of entries.") );
+      ++line;
    }
 
    QPushButton* pButton = new QPushButton( i18n("Test your regular expressions"), page );
@@ -930,13 +959,22 @@ void OptionDialog::setupDiffPage( void )
 
    label = new QLabel( i18n("Irrelevant merge command:"), page );
    gbox->addWidget( label, line, 0 );
-   pLE = new OptionLineEdit( "", "IrrelevantMergeCmd", &m_IrrelevantMergeCmd, page, this );
+   OptionLineEdit* pLE = new OptionLineEdit( "", "IrrelevantMergeCmd", &m_IrrelevantMergeCmd, page, this );
    gbox->addWidget( pLE, line, 1 );
    QToolTip::add( label, i18n("If specified this script is run after automerge\n"
                               "when no other relevant changes were detected.\n"
                               "Called with the parameters: filename1 filename2 filename3") );
    ++line;
 
+
+   OptionCheckBox* pAutoSaveAndQuit = new OptionCheckBox( i18n("Auto save and quit on merge without conflicts"), false,
+      "AutoSaveAndQuitOnMergeWithoutConflicts", &m_bAutoSaveAndQuitOnMergeWithoutConflicts, page, this );
+   gbox->addMultiCellWidget( pAutoSaveAndQuit, line, line, 0, 1 );
+   QToolTip::add( pAutoSaveAndQuit, i18n("When KDiff3 was started for a file-merge from the commandline and all\n" 
+                                         "conflicts are solvable without user interaction then automatically save and quit.\n"
+                                         "(Similar to command line option \"--auto\".") );
+   ++line;
+
    topLayout->addStretch(10);
 }
 
@@ -1050,8 +1088,14 @@ void OptionDialog::setupDirectoryMergePage( void )
    QToolTip::add( pFullAnalysis, i18n("Do a full analysis and show statistics information in extra columns.\n"
                                       "(Slower than a binary comparison, much slower for binary files.)") );
    
-   OptionRadioButton* pTrustDate = new OptionRadioButton( i18n("Trust the modification date (unsafe)"), false, "TrustDate", &m_bDmTrustDate, pBG, this );
+   OptionRadioButton* pTrustDate = new OptionRadioButton( i18n("Trust the size and modification date (unsafe)"), false, "TrustDate", &m_bDmTrustDate, pBG, this );
    QToolTip::add( pTrustDate, i18n("Assume that files are equal if the modification date and file length are equal.\n"
+                                   "Files with equal contents but different modification dates will appear as different.\n"
+                                     "Useful for big directories or slow networks.") );
+   
+   OptionRadioButton* pTrustDateFallbackToBinary = new OptionRadioButton( i18n("Trust the size and date, but use binary comparison if date doesn't match (unsafe)"), false, "TrustDateFallbackToBinary", &m_bDmTrustDateFallbackToBinary, pBG, this );
+   QToolTip::add( pTrustDateFallbackToBinary, i18n("Assume that files are equal if the modification date and file length are equal.\n"
+                                     "If the date isn't equal but the sizes are, use binary comparison.\n"
                                      "Useful for big directories or slow networks.") );
 
    OptionRadioButton* pTrustSize = new OptionRadioButton( i18n("Trust the size (unsafe)"), false, "TrustSize", &m_bDmTrustSize, pBG, this );
@@ -1068,7 +1112,7 @@ void OptionDialog::setupDirectoryMergePage( void )
    ++line;
 
    // Allow white-space only differences to be considered equal
-   OptionCheckBox* pWhiteSpaceDiffsEqual = new OptionCheckBox( i18n("White space differences considered equal"), false,"WhiteSpaceEqual", &m_bDmWhiteSpaceEqual, page, this );
+   OptionCheckBox* pWhiteSpaceDiffsEqual = new OptionCheckBox( i18n("White space differences considered equal"), true,"WhiteSpaceEqual", &m_bDmWhiteSpaceEqual, page, this );
    gbox->addMultiCellWidget( pWhiteSpaceDiffsEqual, line, line, 0, 1 );
    QToolTip::add( pWhiteSpaceDiffsEqual, i18n(
                   "If files differ only by white space consider them equal.\n"
@@ -1113,8 +1157,59 @@ static void insertCodecs(OptionComboBox* p)
 }
 */
 
+// UTF8-Codec that saves a BOM
+class Utf8BOMCodec : public QTextCodec
+{
+   public:
+   const char * name () const { return "UTF-8-BOM"; }
+   int mibEnum () const { return 2123; }
+   int heuristicContentMatch(const char*, int) const { return 0; }
+   class UTF8BOMEncoder : public QTextEncoder
+   {
+      bool bBOMAdded;
+   public:
+      UTF8BOMEncoder()  {  bBOMAdded=false;  }
+      QCString fromUnicode(const QString& uc, int& lenInOut )
+      {
+         QCString r;
+         if (!bBOMAdded)
+         {
+            r += "\xEF\xBB\xBF";
+            bBOMAdded=true;
+         }
+         r += uc.utf8();
+         lenInOut = r.length();
+         return r;
+      }
+   };
+   QTextEncoder* makeEncoder() const
+   {
+      return new UTF8BOMEncoder;
+   }
+
+   class UTF8BOMDecoder : public QTextDecoder
+   {
+      QTextDecoder *m_pDecoder;
+   public:
+      UTF8BOMDecoder() {   m_pDecoder = QTextCodec::codecForName("UTF-8")->makeDecoder();  }
+      ~UTF8BOMDecoder() {
+         delete m_pDecoder;
+      }
+      QString toUnicode( const char* p, int len)
+      {
+         return m_pDecoder->toUnicode( p, len );
+      }
+   };
+   QTextDecoder* makeDecoder() const
+   {
+      return new UTF8BOMDecoder;
+   }
+};
+
 void OptionDialog::setupRegionalPage( void )
 {
+   new Utf8BOMCodec();
+
    QFrame *page = addPage( i18n("Regional Settings"), i18n("Regional Settings"),
                            BarIcon("locale"/*"charset"*/, KIcon::SizeMedium ) );
    QVBoxLayout *topLayout = new QVBoxLayout( page, 5, spacingHint() );
@@ -1277,21 +1372,45 @@ static char* countryMap[]={
    gbox->addWidget( label, line, 0 );
    m_pEncodingAComboBox = new OptionEncodingComboBox( "EncodingForA", &m_pEncodingA, page, this );
    gbox->addWidget( m_pEncodingAComboBox, line, 1 );
+
+   QString autoDetectToolTip = i18n(
+      "If enabled then Unicode (UTF-16 or UTF-8) encoding will be detected.\n"
+      "If the file encoding is not detected then the selected encoding will be used as fallback.\n"
+      "(Unicode detection depends on the first bytes of a file - the byte order mark \"BOM\".)"
+      );
+   m_pAutoDetectUnicodeA = new OptionCheckBox( i18n("Auto Detect Unicode"), true, "AutoDetectUnicodeA", &m_bAutoDetectUnicodeA, page, this );
+   gbox->addWidget( m_pAutoDetectUnicodeA, line, 2 );
+   QToolTip::add( m_pAutoDetectUnicodeA, autoDetectToolTip );
    ++line;
+
    label = new QLabel( i18n("File Encoding for B:"), page );
    gbox->addWidget( label, line, 0 );
    m_pEncodingBComboBox = new OptionEncodingComboBox( "EncodingForB", &m_pEncodingB, page, this );
    gbox->addWidget( m_pEncodingBComboBox, line, 1 );
+   m_pAutoDetectUnicodeB = new OptionCheckBox( i18n("Auto Detect Unicode"), true, "AutoDetectUnicodeB", &m_bAutoDetectUnicodeB, page, this );
+   gbox->addWidget( m_pAutoDetectUnicodeB, line, 2 );
+   QToolTip::add( m_pAutoDetectUnicodeB, autoDetectToolTip );
    ++line;
+
    label = new QLabel( i18n("File Encoding for C:"), page );
    gbox->addWidget( label, line, 0 );
    m_pEncodingCComboBox = new OptionEncodingComboBox( "EncodingForC", &m_pEncodingC, page, this );
    gbox->addWidget( m_pEncodingCComboBox, line, 1 );
+   m_pAutoDetectUnicodeC = new OptionCheckBox( i18n("Auto Detect Unicode"), true, "AutoDetectUnicodeC", &m_bAutoDetectUnicodeC, page, this );
+   gbox->addWidget( m_pAutoDetectUnicodeC, line, 2 );
+   QToolTip::add( m_pAutoDetectUnicodeC, autoDetectToolTip );
    ++line;
+
    label = new QLabel( i18n("File Encoding for Merge Output and Saving:"), page );
    gbox->addWidget( label, line, 0 );
    m_pEncodingOutComboBox = new OptionEncodingComboBox( "EncodingForOutput", &m_pEncodingOut, page, this );
    gbox->addWidget( m_pEncodingOutComboBox, line, 1 );
+   m_pAutoSelectOutEncoding = new OptionCheckBox( i18n("Auto Select"), true, "AutoSelectOutEncoding", &m_bAutoSelectOutEncoding, page, this );
+   gbox->addWidget( m_pAutoSelectOutEncoding, line, 2 );
+   QToolTip::add( m_pAutoSelectOutEncoding, i18n(
+      "If enabled then the encoding from the input files is used.\n"
+      "In ambiguous cases a dialog will ask the user to choose the encoding for saving."
+      ) );
    ++line;
    label = new QLabel( i18n("File Encoding for Preprocessor Files:"), page );
    gbox->addWidget( label, line, 0 );
@@ -1301,6 +1420,8 @@ static char* countryMap[]={
 
    connect(m_pSameEncoding, SIGNAL(toggled(bool)), this, SLOT(slotEncodingChanged()));
    connect(m_pEncodingAComboBox, SIGNAL(activated(int)), this, SLOT(slotEncodingChanged()));
+   connect(m_pAutoDetectUnicodeA, SIGNAL(toggled(bool)), this, SLOT(slotEncodingChanged()));
+   connect(m_pAutoSelectOutEncoding, SIGNAL(toggled(bool)), this, SLOT(slotEncodingChanged()));
 
    OptionCheckBox* pRightToLeftLanguage = new OptionCheckBox( i18n("Right To Left Language"), false, "RightToLeftLanguage", &m_bRightToLeftLanguage, page, this );
    gbox->addMultiCellWidget( pRightToLeftLanguage, line, line, 0, 1 );
@@ -1313,14 +1434,18 @@ static char* countryMap[]={
    topLayout->addStretch(10);
 }
 
+#ifdef _WIN32
+#include "ccInstHelper.cpp"
+#endif
+
 void OptionDialog::setupIntegrationPage( void )
 {
    QFrame *page = addPage( i18n("Integration"), i18n("Integration Settings"),
                            BarIcon("launch"/*"charset"*/, KIcon::SizeMedium ) );
    QVBoxLayout *topLayout = new QVBoxLayout( page, 5, spacingHint() );
 
-   QGridLayout *gbox = new QGridLayout( 3, 2 );
-   gbox->setColStretch(1,5);
+   QGridLayout *gbox = new QGridLayout( 3, 3 );
+   gbox->setColStretch(2,5);
    topLayout->addLayout( gbox );
    int line=0;
 
@@ -1328,7 +1453,7 @@ void OptionDialog::setupIntegrationPage( void )
    label = new QLabel( i18n("Command line options to ignore:"), page );
    gbox->addWidget( label, line, 0 );
    OptionLineEdit* pIgnorableCmdLineOptions = new OptionLineEdit( "-u;-query;-html;-abort", "IgnorableCmdLineOptions", &m_ignorableCmdLineOptions, page, this );
-   gbox->addWidget( pIgnorableCmdLineOptions, line, 1 );
+   gbox->addMultiCellWidget( pIgnorableCmdLineOptions, line, line, 1,2 );
    QToolTip::add( label, i18n(
       "List of command line options that should be ignored when KDiff3 is used by other tools.\n"
       "Several values can be specified if separated via ';'\n"
@@ -1336,9 +1461,46 @@ void OptionDialog::setupIntegrationPage( void )
       ));
    ++line;
 
+#ifdef _WIN32
+   QPushButton* pIntegrateWithClearCase = new QPushButton( i18n("Integrate with ClearCase"), page);
+   gbox->addWidget( pIntegrateWithClearCase, line, 0 );
+   QToolTip::add( pIntegrateWithClearCase, i18n(
+                 "Integrate with Rational ClearCase from IBM.\n"
+                 "Modifies the \"map\" file in ClearCase-subdir \"lib/mgrs\"\n"
+                 "(Only enabled when ClearCase \"bin\" directory is in the path.)"));
+   connect(pIntegrateWithClearCase, SIGNAL(clicked()),this, SLOT(slotIntegrateWithClearCase()) );
+   pIntegrateWithClearCase->setEnabled( integrateWithClearCase( "existsClearCase", "" )!=0 );
+
+   QPushButton* pRemoveClearCaseIntegration = new QPushButton( i18n("Remove ClearCase Integration"), page);
+   gbox->addWidget( pRemoveClearCaseIntegration, line, 1 );
+   QToolTip::add( pRemoveClearCaseIntegration, i18n(
+                 "Restore the old \"map\" file from before doing the Clearcase integration."));
+   connect(pRemoveClearCaseIntegration, SIGNAL(clicked()),this, SLOT(slotRemoveClearCaseIntegration()) );
+   pRemoveClearCaseIntegration->setEnabled( integrateWithClearCase( "existsClearCase", "" )!=0 );
+
+   ++line;
+#endif
+
    topLayout->addStretch(10);
 }
 
+void OptionDialog::slotIntegrateWithClearCase()
+{
+#ifdef _WIN32
+   char kdiff3CommandPath[1000];
+   GetModuleFileNameA( 0, kdiff3CommandPath, sizeof(kdiff3CommandPath)-1 );
+   integrateWithClearCase( "install", kdiff3CommandPath );
+#endif
+}
+
+void OptionDialog::slotRemoveClearCaseIntegration()
+{
+#ifdef _WIN32
+   char kdiff3CommandPath[1000];
+   GetModuleFileNameA( 0, kdiff3CommandPath, sizeof(kdiff3CommandPath)-1 );
+   integrateWithClearCase( "uninstall", kdiff3CommandPath );
+#endif
+}
 
 void OptionDialog::slotEncodingChanged()
 {
@@ -1352,6 +1514,12 @@ void OptionDialog::slotEncodingChanged()
       m_pEncodingOutComboBox->setCurrentItem( m_pEncodingAComboBox->currentItem() );
       m_pEncodingPPComboBox->setEnabled( false );
       m_pEncodingPPComboBox->setCurrentItem( m_pEncodingAComboBox->currentItem() );
+      m_pAutoDetectUnicodeB->setEnabled( false );
+      m_pAutoDetectUnicodeB->setChecked( m_pAutoDetectUnicodeA->isChecked() );
+      m_pAutoDetectUnicodeC->setEnabled( false );
+      m_pAutoDetectUnicodeC->setChecked( m_pAutoDetectUnicodeA->isChecked() );
+      m_pAutoSelectOutEncoding->setEnabled( false );
+      m_pAutoSelectOutEncoding->setChecked( m_pAutoDetectUnicodeA->isChecked() );
    }
    else
    {
@@ -1359,6 +1527,10 @@ void OptionDialog::slotEncodingChanged()
       m_pEncodingCComboBox->setEnabled( true );
       m_pEncodingOutComboBox->setEnabled( true );
       m_pEncodingPPComboBox->setEnabled( true );
+      m_pAutoDetectUnicodeB->setEnabled( true );
+      m_pAutoDetectUnicodeC->setEnabled( true );
+      m_pAutoSelectOutEncoding->setEnabled( true );
+      m_pEncodingOutComboBox->setEnabled( !m_pAutoSelectOutEncoding->isChecked() );
    }
 }
 
@@ -1407,6 +1579,17 @@ void OptionDialog::slotApply( void )
    }
 
    emit applyClicked();   
+
+#ifdef _WIN32
+   QString locale = m_language;
+   if ( locale == "Auto" || locale.isEmpty() )
+      locale = locale = QTextCodec::locale();
+   int spacePos = locale.find(' ');
+   if (spacePos>0) locale = locale.left(spacePos);
+   QSettings settings;
+   settings.setPath("KDiff3", "diff-ext", QSettings::User );
+   settings.writeEntry( "Language", locale );
+#endif
 }
 
 /** Set the default values in the widgets only, while the
