@@ -2,7 +2,7 @@
                          pdiff.cpp  -  Implementation for class KDiff3App
                          ---------------
     begin                : Mon March 18 20:04:50 CET 2002
-    copyright            : (C) 2002-2005 by Joachim Eibl
+    copyright            : (C) 2002-2007 by Joachim Eibl
     email                : joachim.eibl at gmx.de
  ***************************************************************************/
 
@@ -295,11 +295,11 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
 
       // First get all input data.
       pp.setInformation(i18n("Loading A"));
-      m_sd1.readAndPreprocess(m_pOptionDialog->m_pEncodingA);
+      m_sd1.readAndPreprocess(m_pOptionDialog->m_pEncodingA, m_pOptionDialog->m_bAutoDetectUnicodeA );
       pp.step();
 
       pp.setInformation(i18n("Loading B"));
-      m_sd2.readAndPreprocess(m_pOptionDialog->m_pEncodingB);
+      m_sd2.readAndPreprocess(m_pOptionDialog->m_pEncodingB, m_pOptionDialog->m_bAutoDetectUnicodeB );
       pp.step();
    }
    else
@@ -333,7 +333,7 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
       if (bLoadFiles)
       {
          pp.setInformation(i18n("Loading C"));
-         m_sd3.readAndPreprocess(m_pOptionDialog->m_pEncodingC);
+         m_sd3.readAndPreprocess(m_pOptionDialog->m_pEncodingC, m_pOptionDialog->m_bAutoDetectUnicodeC );
          pp.step();
       }
 
@@ -395,6 +395,10 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
 
    m_bTripleDiff = ! m_sd3.isEmpty();
 
+   m_pMergeResultWindowTitle->setEncodings( m_sd1.getEncoding(), m_sd2.getEncoding(), m_sd3.getEncoding() );
+   if ( ! m_pOptionDialog->m_bAutoSelectOutEncoding )
+      m_pMergeResultWindowTitle->setEncoding( m_pOptionDialog->m_pEncodingOut );
+
    if ( bGUI )
    {
       const ManualDiffHelpList* pMDHL = &m_manualDiffHelpList;
@@ -416,9 +420,9 @@ void KDiff3App::init( bool bAuto, TotalDiffStatus* pTotalDiffStatus, bool bLoadF
       m_sd2.getLineDataForDisplay(), m_sd2.getSizeLines(),
       m_bTripleDiff ? m_sd3.getLineDataForDisplay() : 0, m_sd3.getSizeLines(),
       &m_diff3LineList,
-      pTotalDiffStatus,
-      m_outputFilename.isEmpty() ? QString("unnamed.txt") : m_outputFilename
+      pTotalDiffStatus      
       );
+   m_pMergeResultWindowTitle->setFileName( m_outputFilename.isEmpty() ? QString("unnamed.txt") : m_outputFilename );
 
    if ( !bGUI ) 
    {
@@ -666,8 +670,14 @@ void KDiff3App::initView()
    m_pMergeWindowFrame = new QWidget( pVSplitter );
    QHBoxLayout* pMergeHLayout = new QHBoxLayout( m_pMergeWindowFrame,0,0 );
 
+   QVBoxLayout* pMergeVLayout = new QVBoxLayout();
+   pMergeHLayout->addLayout( pMergeVLayout, 1 );
+
+   m_pMergeResultWindowTitle = new WindowTitleWidget(m_pOptionDialog, m_pMergeWindowFrame);
+   pMergeVLayout->addWidget( m_pMergeResultWindowTitle );
+
    m_pMergeResultWindow = new MergeResultWindow( m_pMergeWindowFrame, m_pOptionDialog, statusBar() );
-   pMergeHLayout->addWidget( m_pMergeResultWindow );
+   pMergeVLayout->addWidget( m_pMergeResultWindow, 1 );
 
    m_pMergeVScrollBar = new QScrollBar( Qt::Vertical, m_pMergeWindowFrame );
    pMergeHLayout->addWidget( m_pMergeVScrollBar );
@@ -680,6 +690,7 @@ void KDiff3App::initView()
    pVSplitter->setSizes( sizes );
 
    m_pMergeResultWindow->installEventFilter( this );       // for Cut/Copy/Paste-shortcuts
+   m_pMergeResultWindow->installEventFilter( m_pMergeResultWindowTitle ); // for focus tracking
 
    QHBoxLayout* pHScrollBarLayout = new QHBoxLayout( pVLayout );
    m_pHScrollBar = new ReversibleScrollBar( Qt::Horizontal, m_pMainWidget, &m_pOptionDialog->m_bRightToLeftLanguage );
@@ -720,7 +731,8 @@ void KDiff3App::initView()
    connect( p, SIGNAL( resizeSignal() ),this, SLOT(resizeMergeResultWindow()));
    connect( p, SIGNAL( selectionEnd() ), this, SLOT( slotSelectionEnd() ) );
    connect( p, SIGNAL( newSelection() ), this, SLOT( slotSelectionStart() ) );
-   connect( p, SIGNAL( modified() ), this, SLOT( slotOutputModified() ) );
+   connect( p, SIGNAL( modifiedChanged(bool) ), this, SLOT( slotOutputModified(bool) ) );
+   connect( p, SIGNAL( modifiedChanged(bool) ), m_pMergeResultWindowTitle, SLOT( slotSetModified(bool) ) );
    connect( p, SIGNAL( updateAvailabilities() ), this, SLOT( slotUpdateAvailabilities() ) );
    connect( p, SIGNAL( showPopupMenu(const QPoint&) ), this, SLOT(showPopupMenu(const QPoint&)));
    connect( p, SIGNAL( noRelevantChangesDetected() ), this, SLOT(slotNoRelevantChangesDetected()));
@@ -805,6 +817,8 @@ void KDiff3App::slotAfterFirstPaint()
    else
    {
       m_pMergeResultWindow->slotGoTop();
+      if ( ! m_outputFilename.isEmpty() && ! m_pMergeResultWindow->isUnsolvedConflictAtCurrent() )
+         m_pMergeResultWindow->slotGoNextUnsolvedConflict();
    }
 
    if (m_pCornerWidget)
@@ -867,7 +881,9 @@ bool KDiff3App::eventFilter( QObject* o, QEvent* e )
        int pageSize =  m_DTWHeight;
        switch( k->key() )
        {
-       case Qt::Key_Down:     if (!bCtrl) ++deltaY; break;
+       case Qt::Key_Down:     if (!bCtrl) 
+                                 ++deltaY; 
+                              break;
        case Qt::Key_Up:       if (!bCtrl) --deltaY; break;
        case Qt::Key_PageDown: if (!bCtrl) deltaY+=pageSize; break;
        case Qt::Key_PageUp:   if (!bCtrl) deltaY-=pageSize; break;
@@ -972,13 +988,22 @@ void KDiff3App::slotFileOpen()
 
    for(;;)
    {
-      OpenDialog d(this,
+   
+     OpenDialog d(this,
+         QDir::convertSeparators( m_bDirCompare ? m_pDirectoryMergeWindow->getDirNameA() : m_sd1.isFromBuffer() ? QString("") : m_sd1.getAliasName() ),
+         QDir::convertSeparators( m_bDirCompare ? m_pDirectoryMergeWindow->getDirNameB() : m_sd2.isFromBuffer() ? QString("") : m_sd2.getAliasName() ),
+         QDir::convertSeparators( m_bDirCompare ? m_pDirectoryMergeWindow->getDirNameC() : m_sd3.isFromBuffer() ? QString("") : m_sd3.getAliasName() ),
+         m_bDirCompare ? ! m_pDirectoryMergeWindow->getDirNameDest().isEmpty() : !m_outputFilename.isEmpty(),
+         QDir::convertSeparators( m_bDirCompare ? m_pDirectoryMergeWindow->getDirNameDest() : m_bDefaultFilename ? QString("") : m_outputFilename ),
+         SLOT(slotConfigure()), m_pOptionDialog );
+         
+      /*OpenDialog d(this,
          m_sd1.isFromBuffer() ? QString("") : m_sd1.getAliasName(),
          m_sd2.isFromBuffer() ? QString("") : m_sd2.getAliasName(),
          m_sd3.isFromBuffer() ? QString("") : m_sd3.getAliasName(),
          !m_outputFilename.isEmpty(),
          m_bDefaultFilename ? QString("") : m_outputFilename,
-         SLOT(slotConfigure()), m_pOptionDialog );
+         SLOT(slotConfigure()), m_pOptionDialog );*/
       int status = d.exec();
       if ( status == QDialog::Accepted )
       {
@@ -1463,9 +1488,9 @@ void KDiff3App::slotClipboardChanged()
    //editPaste->setEnabled(!s.isEmpty());
 }
 
-void KDiff3App::slotOutputModified()
+void KDiff3App::slotOutputModified(bool bModified)
 {
-   if ( !m_bOutputModified )
+   if ( bModified && !m_bOutputModified )
    {
       m_bOutputModified=true;
       slotUpdateAvailabilities();
@@ -1664,7 +1689,7 @@ bool KDiff3App::improveFilenames( bool bCreateNewInstance )
             !m_outputFilename.isEmpty()
             );
    
-         m_bDirCompare = true;  // This seems redundant but it might have beed reset during full analysis.
+         m_bDirCompare = true;  // This seems redundant but it might have been reset during full analysis.
    
          if (bSuccess)
          {
