@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2006 by Joachim Eibl                               *
+ *   Copyright (C) 2005-2007 by Joachim Eibl                               *
  *   joachim.eibl at gmx.de                                                *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -252,10 +252,25 @@ void OpenDialog::inputFilenameChanged()
    }
 }
 
+static void fixCurrentText( QComboBox* pCB )
+{
+   QString s = pCB->currentText();
+
+   int pos = s.find( '\n' );
+   if ( pos>=0 )
+      s=s.left(pos);
+   pos = s.find( '\r' );
+   if ( pos>=0 )
+      s=s.left(pos);
+
+   pCB->setCurrentText( s );
+}
+
 void OpenDialog::accept()
 {
    unsigned int maxNofRecentFiles = 10;
 
+   fixCurrentText( m_pLineA );
    QString s = m_pLineA->currentText();
    s = KURL::fromPathOrURL(s).prettyURL();
    QStringList* sl = &m_pOptions->m_recentAFiles;
@@ -264,6 +279,7 @@ void OpenDialog::accept()
    if ( !s.isEmpty() ) sl->prepend( s );
    if (sl->count()>maxNofRecentFiles) sl->erase( sl->at(maxNofRecentFiles), sl->end() );
 
+   fixCurrentText( m_pLineB );
    s = m_pLineB->currentText();
    s = KURL::fromPathOrURL(s).prettyURL();
    sl = &m_pOptions->m_recentBFiles;
@@ -271,6 +287,7 @@ void OpenDialog::accept()
    if ( !s.isEmpty() ) sl->prepend( s );
    if (sl->count()>maxNofRecentFiles) sl->erase( sl->at(maxNofRecentFiles), sl->end() );
 
+   fixCurrentText( m_pLineC );
    s = m_pLineC->currentText();
    s = KURL::fromPathOrURL(s).prettyURL();
    sl = &m_pOptions->m_recentCFiles;
@@ -278,6 +295,7 @@ void OpenDialog::accept()
    if ( !s.isEmpty() ) sl->prepend( s );
    if (sl->count()>maxNofRecentFiles) sl->erase( sl->at(maxNofRecentFiles), sl->end() );
 
+   fixCurrentText( m_pLineOut );
    s = m_pLineOut->currentText();
    s = KURL::fromPathOrURL(s).prettyURL();
    sl = &m_pOptions->m_recentOutputFiles;
